from opengsq.protocols.ase import ASE
from opengsq.protocols.battlefield import Battlefield
from opengsq.protocols.doom3 import Doom3
from opengsq.protocols.eos import EOS
from opengsq.protocols.fivem import FiveM
from opengsq.protocols.gamespy1 import GameSpy1
from opengsq.protocols.gamespy2 import GameSpy2
from opengsq.protocols.gamespy3 import GameSpy3
from opengsq.protocols.gamespy4 import GameSpy4
from opengsq.protocols.killingfloor import KillingFloor
from opengsq.protocols.minecraft import Minecraft
from opengsq.protocols.quake1 import Quake1
from opengsq.protocols.quake2 import Quake2
from opengsq.protocols.quake3 import Quake3
from opengsq.protocols.raknet import RakNet
from opengsq.protocols.samp import Samp
from opengsq.protocols.satisfactory import Satisfactory
from opengsq.protocols.scum import Scum
from opengsq.protocols.source import Source
from opengsq.protocols.teamspeak3 import TeamSpeak3
from opengsq.protocols.unreal2 import Unreal2
from opengsq.protocols.vcmp import Vcmp
from opengsq.protocols.won import WON



from opengsq.responses.killingfloor import Status
from opengsq.binary_reader import BinaryReader
from opengsq.exceptions import InvalidPacketException
from opengsq.protocol_socket import UdpClient
from opengsq.protocols.unreal2 import Unreal2


class KillingFloor(Unreal2):
    """
    This class represents the Killing Floor Protocol. It provides methods to interact with the Killing Floor API.
    """

    full_name = "Killing Floor Protocol"

    async def get_details(self) -> Status:
        """
        Asynchronously retrieves the details of the game server.

        :return: A Status object containing the details of the game server.
        """
        response = await UdpClient.communicate(
            self, b"\x79\x00\x00\x00" + bytes([self._DETAILS])
        )

        # Remove the first 4 bytes \x80\x00\x00\x00
        br = BinaryReader(response[4:])
        header = br.read_byte()

        if header != self._DETAILS:
            raise InvalidPacketException(
                "Packet header mismatch. Received: {}. Expected: {}.".format(
                    chr(header), chr(self._DETAILS)
                )
            )

        return Status(
            server_id=br.read_long(),
            server_ip=br.read_string(),
            game_port=br.read_long(),
            query_port=br.read_long(),
            server_name=self._read_string(br),
            map_name=self._read_string(br),
            game_type=self._read_string(br),
            num_players=br.read_long(),
            max_players=br.read_long(),
            wave_current=br.read_long(),
            wave_total=br.read_long(),
            ping=br.read_long(),
            flags=br.read_long(),
            skill=self._read_string(br),
        )


if __name__ == "__main__":
    import asyncio

    async def main_async():
        # killingfloor
        killingFloor = KillingFloor(host="185.80.128.168", port=7708, timeout=10.0)
        details = await killingFloor.get_details()
        print(details)
        rules = await killingFloor.get_rules()
        print(rules)
        players = await killingFloor.get_players()
        print(players)

    asyncio.run(main_async())



from __future__ import annotations

from typing import Any
import aiohttp
import time

from opengsq.protocol_base import ProtocolBase


class FiveM(ProtocolBase):
    """
    This class represents the FiveM Protocol (https://docs.fivem.net/docs/server-manual/proxy-setup/). It provides methods to interact with the FiveM API.
    """

    full_name = "FiveM Protocol"

    async def _get(self, filename: str) -> dict[str, Any]:
        """
        Asynchronously retrieves the JSON data from the given filename on the server.

        :param filename: The filename to retrieve data from.
        :return: A dictionary containing the JSON data.
        """
        url = f"http://{self._host}:{self._port}/{filename}.json?v={int(time.time())}"

        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                return await response.json(content_type=None)

    async def get_info(self) -> dict[str, Any]:
        """
        Asynchronously retrieves the information of the game server.

        :return: A dictionary containing the information of the game server.
        """
        return await self._get("info")

    async def get_players(self) -> list[dict[str, Any]]:
        """
        Asynchronously retrieves the list of players on the game server.

        :return: A list of players on the game server.
        """
        return await self._get("players")

    async def get_dynamic(self) -> dict[str, Any]:
        """
        Asynchronously retrieves the dynamic data of the game server.

        :return: A dictionary containing the dynamic data of the game server.
        """
        return await self._get("dynamic")


if __name__ == "__main__":
    import asyncio

    async def main_async():
        fivem = FiveM(host="144.217.10.12", port=30120, timeout=5.0)
        info = await fivem.get_info()
        print(info)
        players = await fivem.get_players()
        print(players)
        dynamic = await fivem.get_dynamic()
        print(dynamic)

    asyncio.run(main_async())



from dataclasses import dataclass


@dataclass
class Status:
    """
    Represents the status response from a Minecraft server.
    """

    edition: str
    """The edition of the server (MCPE or MCEE for Education Edition)."""

    motd_line1: str
    """The first line of the Message of the Day (MOTD)."""

    protocol_version: int
    """The protocol version of the server."""

    version_name: str
    """The version name of the server."""

    num_players: int
    """The number of players currently on the server."""

    max_players: int
    """The maximum number of players that can join the server."""

    server_unique_id: str
    """The unique ID of the server."""

    motd_line2: str
    """The second line of the Message of the Day (MOTD)."""

    game_mode: str
    """The game mode of the server."""

    game_mode_numeric: int
    """The numeric representation of the game mode."""

    port_ipv4: int
    """The IPv4 port of the server."""

    port_ipv6: int
    """The IPv6 port of the server."""



import argparse
import asyncio
import inspect
import json
import os
import re
import sys
from functools import partial
from pydoc import locate
from typing import Mapping, Sequence

from opengsq.protocol_base import ProtocolBase
from opengsq.version import __version__


class CLI:
    def __init__(self):
        self.__paths = {}

    def register(self, parser: argparse.ArgumentParser):
        # Add version argument
        parser.add_argument(
            "-V",
            "--version",
            action="store_true",
            help="print the opengsq version number and exit",
        )

        opengsq_path = os.path.abspath(os.path.dirname(__file__))
        subparsers = parser.add_subparsers(dest="subparser_name")
        pattern = re.compile(r"from\s+(\S+)\s+import\s+(.+,?\S)")

        # Load all protocols from __init__.py
        with open(os.path.join(opengsq_path, "protocols", "__init__.py")) as f:
            for protocol_path, protocol_classnames in re.findall(pattern, f.read()):
                for protocol_classname in protocol_classnames.split(","):
                    name, fullpath, parameters = self.__extract(
                        protocol_path, protocol_classname
                    )

                    # Save to self.__paths dictionary
                    # Example: name = 'source', fullpath = 'opengsq.protocols.source.Source'
                    self.__paths[name] = fullpath

                    # Add parser and arguments
                    obj: ProtocolBase = locate(fullpath)
                    sub = subparsers.add_parser(name, help=obj.full_name)
                    self.__add_arguments(sub, parameters)

                    method_names = [
                        func
                        for func in dir(obj)
                        if callable(getattr(obj, func)) and func.startswith("get_")
                    ]

                    sub.add_argument(
                        "--function",
                        default=method_names[0],
                        type=str,
                        help="(default: %(default)s)",
                    )
                    sub.add_argument("--indent", default=None, type=int, nargs="?")

    # Get the query response in json format
    async def run(self, args: Sequence[str]) -> str:
        # Return version if -V or --version
        if args.version:
            return "v" + __version__
        else:
            del args.version

        # Load the obj from path
        obj = locate(self.__paths[args.subparser_name])
        del args.subparser_name

        function = args.function
        del args.function

        indent = args.indent
        del args.indent

        # Bind values to obj parameters
        for value in vars(args).values():
            obj = partial(obj, value)

        # Create obj()
        protocol: ProtocolBase = obj()
        func = getattr(protocol, function)

        return json.dumps(
            await func(),
            ensure_ascii=False,
            indent=indent,
            default=lambda o: o.__dict__,
        )

    # Extract name, fullpath, parameters from path, classname
    def __extract(self, path: str, classname: str):
        name = path.split(".")[-1]
        fullpath = "{}.{}".format(path, classname.strip())
        parameters = inspect.signature(locate(fullpath).__init__).parameters

        return name, fullpath, parameters

    def __add_arguments(
        self, sub: argparse.ArgumentParser, parameters: Mapping[str, inspect.Parameter]
    ):
        for key in parameters:
            if parameters[key].name == "self":
                continue

            name_or_flags = "--{}".format(parameters[key].name)
            required = parameters[key].default == inspect._empty
            default = None if required else parameters[key].default

            type_mapping = {"str": str, "int": int, "float": float}
            type_func = type_mapping.get(
                parameters[key].annotation, parameters[key].annotation
            )

            help = None if required else "(default: %(default)s)"

            sub.add_argument(
                name_or_flags,
                default=default,
                required=required,
                type=type_func,
                help=help,
            )


def main():
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main_async())
    loop.close()


async def main_async():
    cli = CLI()

    parser = argparse.ArgumentParser()
    cli.register(parser)

    if len(sys.argv) <= 1:
        parser.print_help(sys.stderr)
        sys.exit(-1)

    try:
        args = parser.parse_args()
        result = await cli.run(args)
        sys.stdout.write(result)
    except asyncio.exceptions.TimeoutError:
        sys.stderr.write("opengsq: error: timed out\n")
        sys.exit(-2)

    sys.exit(0)


if __name__ == "__main__":
    main()



from .environment import Environment
from .extra_data_flag import ExtraDataFlag
from .gold_source_info import GoldSourceInfo
from .partial_info import PartialInfo
from .player import Player
from .server_type import ServerType
from .source_info import SourceInfo
from .vac import VAC
from .visibility import Visibility



from dataclasses import dataclass
from ..unreal2 import Status as Unreal2Status


@dataclass
class Status(Unreal2Status):
    """
    Represents the status of a server.
    """

    wave_current: int
    """The current wave number in a game."""

    wave_total: int
    """The total number of waves in a game."""



import asyncio
import socket
from enum import Enum, auto

from opengsq.protocol_base import ProtocolBase


class SocketKind(Enum):
    SOCK_STREAM = auto()
    SOCK_DGRAM = auto()


class Socket():
    @staticmethod
    async def gethostbyname(hostname: str):
        return await asyncio.get_running_loop().run_in_executor(None, socket.gethostbyname, hostname)

    def __init__(self, kind: SocketKind):
        self.__timeout = None
        self.__transport = None
        self.__protocol = None
        self.__kind = kind

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def settimeout(self, value: float):
        self.__timeout = value

    async def connect(self, remote_addr):
        await asyncio.wait_for(self.__connect(remote_addr), timeout=self.__timeout)

    async def __connect(self, remote_addr):
        loop = asyncio.get_running_loop()
        self.__protocol = self.__Protocol(self.__timeout)

        if self.__kind == SocketKind.SOCK_STREAM:
            self.__transport, _ = await loop.create_connection(
                lambda: self.__protocol,
                host=remote_addr[0],
                port=remote_addr[1],
            )
        else:
            self.__transport, _ = await loop.create_datagram_endpoint(
                lambda: self.__protocol,
                remote_addr=remote_addr,
            )

    def close(self):
        if self.__transport:
            self.__transport.close()

    def send(self, data: bytes):
        if self.__kind == SocketKind.SOCK_STREAM:
            self.__transport.write(data)
        else:
            self.__transport.sendto(data)

    async def recv(self) -> bytes:
        return await self.__protocol.recv()

    class __Protocol(asyncio.Protocol):
        def __init__(self, timeout: float):
            self.__packets = asyncio.Queue()
            self.__timeout = timeout

        async def recv(self):
            return await asyncio.wait_for(self.__packets.get(), timeout=self.__timeout)

        def connection_made(self, transport):
            pass

        def connection_lost(self, exc):
            pass

        # Streaming Protocols
        def data_received(self, data):
            """Called when some data is received. data is a non-empty bytes object containing the incoming data."""
            self.__packets.put_nowait(data)

        # Streaming Protocols
        def eof_received(self):
            """
            Called when the other end signals it won't send any more data
            (for example by calling transport.write_eof(), if the other end also uses asyncio).
            """
            pass

        # Datagram Protocols
        def datagram_received(self, data, addr):
            """Called when some datagram is received."""
            self.__packets.put_nowait(data)

        # Datagram Protocols
        def error_received(self, exc):
            """Called when a previous send or receive operation raises an OSError. exc is the OSError instance."""
            pass


class UdpClient(Socket):
    @staticmethod
    async def communicate(protocol: ProtocolBase, data: bytes):
        with UdpClient() as udpClient:
            udpClient.settimeout(protocol._timeout)
            await udpClient.connect((protocol._host, protocol._port))
            udpClient.send(data)
            return await udpClient.recv()

    def __init__(self):
        super().__init__(SocketKind.SOCK_DGRAM)


class TcpClient(Socket):
    @staticmethod
    async def communicate(protocol: ProtocolBase, data: bytes):
        with TcpClient() as tcpClient:
            tcpClient.settimeout(protocol._timeout)
            await tcpClient.connect((protocol._host, protocol._port))
            tcpClient.send(data)
            return await tcpClient.recv()

    def __init__(self):
        super().__init__(SocketKind.SOCK_STREAM)


if __name__ == '__main__':
    async def test_socket_async():
        with Socket() as socket_async:
            socket_async.settimeout(5)
            await socket_async.connect(('122.128.109.245', 27015))
            socket_async.send(
                b'\xFF\xFF\xFF\xFFTSource Engine Query\x00\xFF\xFF\xFF\xFF')
            print(await socket_async.recv())

    loop = asyncio.get_event_loop()
    loop.run_until_complete(test_socket_async())
    loop.close()



from __future__ import annotations

import re
from typing import Any

from opengsq.responses.unreal2 import Player, Status
from opengsq.binary_reader import BinaryReader
from opengsq.exceptions import InvalidPacketException
from opengsq.protocol_base import ProtocolBase
from opengsq.protocol_socket import UdpClient


class Unreal2(ProtocolBase):
    """
    The Unreal2 class inherits from the ProtocolBase class and represents the Unreal 2 Protocol.
    It provides methods to get details, rules, and players from a server using the Unreal 2 Protocol.
    """

    full_name = "Unreal 2 Protocol"

    _DETAILS = 0x00
    _RULES = 0x01
    _PLAYERS = 0x02

    async def get_details(self) -> Status:
        """
        Asynchronously gets the details of a server.

        Returns:
            Status: A Status object containing the details of the server.
        """
        response = await UdpClient.communicate(
            self, b"\x79\x00\x00\x00" + bytes([self._DETAILS])
        )

        # Remove the first 4 bytes \x80\x00\x00\x00
        br = BinaryReader(response[4:])
        header = br.read_byte()

        if header != self._DETAILS:
            raise InvalidPacketException(
                "Packet header mismatch. Received: {}. Expected: {}.".format(
                    chr(header), chr(self._DETAILS)
                )
            )

        return Status(
            server_id=br.read_long(),
            server_ip=br.read_string(),
            game_port=br.read_long(),
            query_port=br.read_long(),
            server_name=self._read_string(br),
            map_name=self._read_string(br),
            game_type=self._read_string(br),
            num_players=br.read_long(),
            max_players=br.read_long(),
            ping=br.read_long(),
            flags=br.read_long(),
            skill=self._read_string(br),
        )

    async def get_rules(self) -> dict[str, Any]:
        """
        Asynchronously gets the rules of a server.

        Returns:
            dict: A dictionary containing the rules of the server.
        """
        response = await UdpClient.communicate(
            self, b"\x79\x00\x00\x00" + bytes([self._RULES])
        )

        # Remove the first 4 bytes \x80\x00\x00\x00
        br = BinaryReader(response[4:])
        header = br.read_byte()

        if header != self._RULES:
            raise InvalidPacketException(
                "Packet header mismatch. Received: {}. Expected: {}.".format(
                    chr(header), chr(self._RULES)
                )
            )

        rules = {}
        rules["Mutators"] = []

        while not br.is_end():
            key = self._read_string(br)
            val = self._read_string(br)

            if key.lower() == "mutator":
                rules["Mutators"].append(val)
            else:
                rules[key] = val

        return rules

    async def get_players(self) -> list[Player]:
        """
        Asynchronously gets the players of a server.

        Returns:
            list: A list of Player objects representing the players of the server.
        """
        response = await UdpClient.communicate(
            self, b"\x79\x00\x00\x00" + bytes([self._PLAYERS])
        )

        # Remove the first 4 bytes \x80\x00\x00\x00
        br = BinaryReader(response[4:])
        header = br.read_byte()

        if header != self._PLAYERS:
            raise InvalidPacketException(
                "Packet header mismatch. Received: {}. Expected: {}.".format(
                    chr(header), chr(self._PLAYERS)
                )
            )

        players = []

        while not br.is_end():
            player = Player(
                id=br.read_long(),
                name=self._read_string(br),
                ping=br.read_long(),
                score=br.read_long(),
                stats_id=br.read_long(),
            )
            players.append(player)

        return players

    @staticmethod
    def strip_colors(text: str) -> str:
        """
        Strips color codes from a string.

        Args:
            text (str): The string to strip color codes from.

        Returns:
            str: The string with color codes stripped.
        """
        return re.compile("\x1b...|[\x00-\x1a]").sub("", text)

    def _read_string(self, br: BinaryReader):
        """
        Reads a string from a BinaryReader object.

        Args:
            br (BinaryReader): The BinaryReader object to read the string from.

        Returns:
            str: The string read from the BinaryReader object.
        """
        length = br.read_byte()
        encoding = "utf-8"

        if length >= 128:
            length = (length & 0x7F) * 2
            encoding = "utf-16"

        return Unreal2.strip_colors(br.read_bytes(length).decode(encoding, "ignore"))


if __name__ == "__main__":
    import asyncio

    async def main_async():
        # ut2004
        unreal2 = Unreal2(host="109.230.224.189", port=6970)
        details = await unreal2.get_details()
        print(details)
        rules = await unreal2.get_rules()
        print(rules)
        players = await unreal2.get_players()
        print(players)

    asyncio.run(main_async())



from dataclasses import dataclass
from .server_type import ServerType
from .environment import Environment
from .visibility import Visibility
from .vac import VAC


@dataclass
class PartialInfo:
    """
    A2S_INFO Partial Info
    """

    protocol: int
    """Protocol version used by the server."""

    name: str
    """Name of the server."""

    map: str
    """Map the server has currently loaded."""

    folder: str
    """Name of the folder containing the game files."""

    game: str
    """Full name of the game."""

    players: int
    """Number of players on the server."""

    max_players: int
    """Maximum number of players the server reports it can hold."""

    bots: int
    """Number of bots on the server."""

    server_type: ServerType
    """Indicates the type of server."""

    environment: Environment
    """Indicates the operating system of the server."""

    visibility: Visibility
    """Indicates whether the server requires a password."""

    vac: VAC
    """Specifies whether the server uses VAC."""



from dataclasses import dataclass


@dataclass
class Status:
    """
    Represents the status response.
    """

    state: int
    """The state."""

    version: int
    """The version."""

    beacon_port: int
    """The beacon port."""



from dataclasses import dataclass


@dataclass
class Player:
    """
    Represents a player in the game.
    """

    id: int
    """The ID of the player."""

    name: str
    """The name of the player."""

    ping: int
    """The ping of the player."""

    score: int
    """The score of the player."""

    stats_id: int
    """The stats ID of the player."""



from dataclasses import dataclass


@dataclass
class Status:
    """
    Represents the response status of a server.
    """

    ip: str
    """The IP address of the server."""

    port: int
    """The port number of the server."""

    name: str
    """The name of the server."""

    num_players: int
    """The number of players currently connected to the server."""

    max_players: int
    """The maximum number of players that can connect to the server."""

    time: int
    """The server time."""

    password: bool
    """A value indicating whether a password is required to connect to the server."""

    version: str
    """The version of the server."""



import os

from setuptools import find_packages, setup

current_dir = os.path.abspath(os.path.dirname(__file__))
version_contents = {}

with open(os.path.join(current_dir, 'opengsq', 'version.py'), encoding='utf-8') as f:
    exec(f.read(), version_contents)

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

try:
    with open('requirements.txt', 'r', encoding='utf-8') as f:
        install_requires = f.read().splitlines()
except FileNotFoundError:
    with open(os.path.join(current_dir, 'opengsq.egg-info', 'requires.txt'), 'r', encoding='utf-8') as f:
        install_requires = f.read().splitlines()

setup(
    name='opengsq',
    version=version_contents['__version__'],
    description='🐍 OpenGSQ - Python library for querying game servers',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=install_requires,
    entry_points={'console_scripts': ['opengsq=opengsq.cli:main']},
    packages=find_packages(exclude=['tests', 'tests.*']),
    python_requires='>=3.7',
    url='https://github.com/opengsq/opengsq-python',
    project_urls={
        'Bug Tracker': 'https://github.com/opengsq/opengsq-python/issues',
        'Source Code': 'https://github.com/opengsq/opengsq-python',
    },
    license='MIT',
    author='OpenGSQ',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from __future__ import annotations

from dataclasses import dataclass
from typing import Optional


@dataclass
class Player:
    """
    Represents a player in the game.
    """

    name: Optional[str] = None
    """The name of the player."""

    team: Optional[str] = None
    """The team of the player."""

    skin: Optional[str] = None
    """The skin of the player."""

    score: Optional[int] = None
    """The score of the player."""

    ping: Optional[int] = None
    """The ping of the player."""

    time: Optional[int] = None
    """The time of the player."""



from dataclasses import dataclass
from typing import Optional


@dataclass
class Player:
    """
    Player Data
    """

    name: str
    """Player Name"""

    score: int
    """Player Score"""

    duration: float
    """Player Duration"""

    deaths: Optional[int] = None
    """Player Deaths"""

    money: Optional[int] = None
    """Player Money"""
